<?php

/*
 * Plugin name: Displet RETS / IDX Plugin
 * Plugin URI: http://displet.com/wordpress-plugins/displet-rets-idx-plugin/
 * Description: RETS/IDX Plugin that inserts real estate listings, statistics, maps, and quick searches into Wordpress pages & widget ready sidebars.
 * Version: 1.5.1.2
 * Author: Displet
 * Author URI: http://displet.com/
 * License: GPL2
 */

/*  Copyright 2011 Displet (email : dev@displet.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2, as 
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

// Register defaults on activation if none exist
function displet_reader_wordpress_plugin_register_defaults(){
	$property_types = array('House', 'Condo', 'Lease', 'Land', 'Multi', 'Ranch');
	foreach ($property_types as $property_type) {
		$displet_rets_idx_options['displet_rets_idx_option_qs_property_types'][$property_type] = 'true';
	}
	$options = array(
		'upgrade' => 0,
		'oodle_region' => 'usa',
		'oodle_category' => 'housing/sale',
		'listings_orientation' => 'gallery',
		'listings_sort' => 'price-descending',
		'listings_per_page' => '10',
		'no_link' => 0,
		'color_scheme' => 'default',
		'quick_search_quick_terms' => 1,
		'quick_search_property_type' => 1,
		'quick_search_price' => 1,
		'quick_search_beds_baths' => 1,
		'quick_search_keywords' => 1,
		'qs_min_price' => '100',
		'qs_max_price' => '1000',
		'qs_price_increment' => '50',
		'qs_beds_min' => '1',
		'qs_beds_max' => '6',
		'qs_baths_min' => '1',
		'qs_baths_max' => '6',
		'qs_min_square_feet' => '500',
		'qs_max_square_feet' => '5000',
		'qs_square_feet_increment' => '500',
		'map_variance' => 2
	);
	foreach ($options as $option_name => $option_value) {
		$displet_rets_idx_options['displet_rets_idx_option_' . $option_name] = $option_value;
	}
	add_option('displet_rets_idx_options', $displet_rets_idx_options);
	// Activation redirect and message options
	add_option('displetreader_wordpress_plugin_do_activation_redirect', true);
	add_option('displetreader_wordpress_plugin_do_activation_message', true);
	add_option('displet_rets_idx_do_activation_message', true);
}
register_activation_hook(__FILE__, 'displet_reader_wordpress_plugin_register_defaults');

// Get Displet RETS/IDX (new) options
$displet_rets_idx_options = get_option('displet_rets_idx_options');
$upgrade_to_idx = $displet_rets_idx_options['displet_rets_idx_option_upgrade'];

// Get DispletReader (legacy) options
$displetreader_options = get_option('displetreader');

// Get DispletReader WordPress Plugin options (base)
$displetreader_wordpress_plugin_options = get_option('displetreader_wordpress_plugin_options');
$use_freemium = $displetreader_wordpress_plugin_options['displetreader_wordpress_plugin_option_displet_freemium'];

// If base option updated, update DispletReader options
if (!empty($displetreader_wordpress_plugin_options['displetreader_wordpress_plugin_option_displet_url'])) {
	if (substr($displetreader_wordpress_plugin_options['displetreader_wordpress_plugin_option_displet_url'], -1) !== '/') {
		$displetreader_wordpress_plugin_options['displetreader_wordpress_plugin_option_displet_url'] .= '/';
	}
	$displetreader_options['displet_url'] = $displetreader_wordpress_plugin_options['displetreader_wordpress_plugin_option_displet_url'];
	update_option('displetreader', $displetreader_options);
}

$displet_url = $displetreader_options['displet_url'];

// If already have a Displet IDX URL saved
if (!empty($displet_url)) {
	require_once 'displetreader-wordpress-plugin/DispletReader.php';
	// Reset base Displet URL to allow removal (loops otherwise)
	$displetreader_wordpress_plugin_options['displetreader_wordpress_plugin_option_displet_url'] = '';
	// Reset Displet WordPress Plugin freemium
	$displetreader_wordpress_plugin_options['displetreader_wordpress_plugin_option_displet_freemium'] = 0;
	// Reset Displet WordPress Plugin activated fremium
	$displetreader_wordpress_plugin_options['displetreader_wordpress_plugin_option_displet_activated_freemium'] = false;
	update_option('displetreader_wordpress_plugin_options', $displetreader_wordpress_plugin_options);
}
// Checked upgrade box from Freemium
else if ($upgrade_to_idx == 1 && $use_freemium == 1) {
	// Reset Displet RETS/IDX upgrade to allow removal (loops otherwise)
	$displet_rets_idx_options['displet_rets_idx_option_upgrade'] = 0;
	update_option('displet_rets_idx_options', $displet_rets_idx_options);
	// Reset Displet WordPress Plugin freemium to allow removal (loops otherwise)
	$displetreader_wordpress_plugin_options['displetreader_wordpress_plugin_option_displet_freemium'] = 0;
	update_option('displetreader_wordpress_plugin_options', $displetreader_wordpress_plugin_options);
	require_once 'displetreader-wordpress-plugin-settings.php';
}
// If no account and have checked freemium on base options page
else if ($use_freemium == 1) {
	// Register default page for Property Details & Search Results
	function displet_rets_idx_register_default_page (){
	    $has_default_page = get_page_by_path('rets');
	    if (!$has_default_page) {    
	        $displet_default_rets_idx_page = array(
	          'post_name' => 'rets',
	          'post_title' => 'Real Estate',
	          'post_type' => 'page',
	          'post_status' => 'publish'
	        );
	        wp_insert_post($displet_default_rets_idx_page);
	    }
	    $default_page = get_page_by_path('rets');
	    $displet_rets_idx_options = get_option('displet_rets_idx_options');
	    $displet_rets_idx_options['displet_rets_idx_option_property_details_page_id'] = $default_page->ID;
	    $displet_rets_idx_options['displet_rets_idx_option_search_results_page_id'] = $default_page->ID;
	    update_option('displet_rets_idx_options', $displet_rets_idx_options);
	}
	if (!isset($displetreader_wordpress_plugin_options['displetreader_wordpress_plugin_option_displet_activated_freemium']) || $displetreader_wordpress_plugin_options['displetreader_wordpress_plugin_option_displet_activated_freemium'] == false) {
		add_action('init', 'displet_rets_idx_register_default_page');
		$displetreader_wordpress_plugin_options['displetreader_wordpress_plugin_option_displet_activated_freemium'] = true;
		update_option('displetreader_wordpress_plugin_options', $displetreader_wordpress_plugin_options);
	}
	require_once 'rets-idx/displet-rets-idx.php';
	// Admin notice
	function displet_retx_idx_admin_message(){
		if (get_option('displet_rets_idx_do_activation_message', false)) {
			delete_option('displet_rets_idx_do_activation_message');
	    	echo '<div class="updated">
	    			<p>Thanks for activating the free version of DispletPress! Set your default settings here or <a href="http://displet.com">get started right away</a>.</p>
	    		  </div>';
	    }
	}
	add_action('admin_notices', 'displet_retx_idx_admin_message');
}
// Displet base options page
else{
	require_once 'displetreader-wordpress-plugin-settings.php';
	// Redirect to admin page on activation
	function my_plugin_redirect() {
	    if (get_option('displetreader_wordpress_plugin_do_activation_redirect', false)) {
	        delete_option('displetreader_wordpress_plugin_do_activation_redirect');
	        wp_redirect(admin_url('admin.php?page=displetreader-uid-slug-submenu')); exit;
	    }
	}
	add_action('admin_init', 'my_plugin_redirect');
	// Admin notice
	function displet_reader_wordpress_plugin_admin_message(){
		if (get_option('displetreader_wordpress_plugin_do_activation_message', false)) {
			delete_option('displetreader_wordpress_plugin_do_activation_message');
	    	echo '<div class="updated">
	    			<p>Thanks for activating DispletPress! Please select your service level below to get started.</p>
	    		  </div>';
	    }
	}
	add_action('admin_notices', 'displet_reader_wordpress_plugin_admin_message');
}

?>