<?php

class DispletRetsIdxQuery{
	public function get_query_url($model) {
		// Retrieve options from settings page
		$displet_rets_idx_option = DispletRetsIdxSettings::get_option();

		// Listings per page defaults to integer
		if (!is_numeric($model['listings_per_page'])) {
			$model['listings_per_page'] = 10;
		}

		// Displet API
		if (!empty($displet_rets_idx_option['displet_rets_idx_option_displet_app_key'])) {
			// Default Displet API query page & listings per page
			$query_url = 'http://staging.api.displet.com/residentials/search?authorization_token=' . $displet_rets_idx_option['displet_rets_idx_option_displet_app_key'];
			$query_url .= '&page=1&limit=' . $model['listings_per_page'];

			// Return fields
			if (!empty($model['return_fields'])) {
				$query_url .=  '&returnFields=' . $model['return_fields'];
			}

			// Sorting
			if ($model['default_sort'] == 'price-descending') {
				$sortby = 'desc';
			}
			else{
				$sortby = 'asc';
			}
			$query_url .= '&priceSorter=' . $sortby;

			// Add to query from model
			$model_key_param_pairs = array(
				'minlistprice' => 'minListPrice',
				'maxlistprice' => 'maxListPrice',
				'minbedrooms' => 'minBedrooms',
				'maxbedrooms' => 'maxBedrooms',
				'minbathrooms' => 'minBathrooms',
				'maxbathrooms' => 'maxBathrooms',
				'minsquarefeet' => 'minSquareFeet',
				'maxsquarefeet' => 'maxSquareFeet',
				'area' => 'area',
				'area_mls_defined' => 'area_mls_defined',
				'city' => 'city',
				'property_type' => 'property_type',
				'minstories' => 'minStories',
				'waterfront' => 'waterfront',
				'pool_on_property' => 'pool_on_property', 
				'subdivision' => 'subdivision',
				'keyword' => 'keyword',
				'school' => 'school',
				'school_district' => 'school_district',
				'minacres' => 'minAcres',
				'zip' => 'zip',
				'is_foreclosure' => 'is_foreclosure',
				'yearbuilt' => 'yearBuilt',
				'status' => 'status',
				'short_sale' => 'short_sale',
				'last_modified' => 'last_modified',
				'listing_agent_id' => 'listing_agent_id',
				'listing_office_id' => 'listing_office_id',
				'min_lot_size' => 'min_lot_size',
				'is_gated_community' => 'is_gated_community',
				'min_sold_date' => 'min_sold_date',
				'max_sold_date' => 'max_sold_date',
				'master_on_main' => 'master_on_main',
				'basement' => 'basement',
				'min_square_feet_available' => 'min_square_feet_available',
				'max_square_feet_available' => 'max_square_feet_available',
				'min_price_per_sq_feet' => 'min_price_per_sq_feet',
				'max_price_per_sq_feet' => 'max_price_per_sq_feet'
			);
			foreach ($model_key_param_pairs as $model_key => $parameter) {
				if (!empty($model[$model_key])) {
					$query_url .= '&' . $parameter . '=' . str_replace(' ', '%20', $model[$model_key]);
				}	
			}

		}
		// Free version API
		else {
			$query_url = 'http://api.oodle.com/api/v2/listings?key=579B95506556&format=json&jsoncallback=none&num_images=1&image_sizes=m&start=1';

			// Listings per page
			if (!is_numeric($model['listings_per_page'])) {
				$model['listings_per_page'] = 10;
			}
			$query_url .= '&num=' . $model['listings_per_page'];

			// Sorting
			if ($model['default_sort'] == 'price-descending') {
				$sortby = 'price_reverse';
			}
			else if ($model['default_sort'] == 'newest') {
				$sortby = 'ctime_reverse';
			}
			else if ($model['default_sort'] == 'oldest') {
				$sortby = 'ctime';
			}
			else{
				$sortby = 'price';
			}
			$query_url .= '&sort=' . $sortby;

			// Translations - make an Oodle URL query parameter in the form of $model[url_parameter] from each existing $model[parameter] so that each $model[parameter] = url_parameter for '&url_parameter='
			if (!empty($model['minlistprice']) && !empty($model['maxlistprice'])) {
				$model['price'] = (intval($model['minlistprice']) * 1000) . '_' . (intval($model['maxlistprice']) * 1000);
			}
			else if (!empty($model['minlistprice'])) {
				$model['price'] = (intval($model['minlistprice']) * 1000) . '+';
			}
			else if (!empty($model['maxlistprice'])) {
				$model['price'] = (intval($model['maxlistprice']) * 1000) . '-';
			}
			if (!empty($model['minbedrooms'])) {
				$model['bedrooms'] = $model['minbedrooms'] . '+';
			}
			if (!empty($model['maxbedrooms'])) {
				$model['bedrooms'] = $model['maxbedrooms'] . '-';
			}
			if (!empty($model['minbathrooms'])) {
				$model['bathrooms'] = $model['minbathrooms'] . '+';
			}
			if (!empty($model['maxbathrooms'])) {
				$model['bathrooms'] = $model['maxbathrooms'] . '-';
			}
			if (!empty($model['minsquarefeet']) && !empty($model['maxsquarefeet'])) {
				$model['square_feet'] = $model['minsquarefeet'] . '_' . $model['maxsquarefeet'];
			}
			else if (!empty($model['minsquarefeet'])) {
				$model['square_feet'] = $model['minsquarefeet'] . '+';
			}
			else if (!empty($model['maxsquarefeet'])) {
				$model['square_feet'] = $model['maxsquarefeet'] . '-';
			}
			if (!empty($model['area'])) {
				/* Prefers to have nationwide region instead of default if location is queried, but need to adapt for Canada
				$model['region'] = 'usa';
				*/
				$model['location'] = $model['area'] . '&radius=0';
			}
			if (!empty($model['area_mls_defined'])) {
				/* Prefers to have nationwide region instead of default if location is queried, but need to adapt for Canada
				$model['region'] = 'usa';
				*/
				$model['location'] = $model['area_mls_defined'] . '&radius=0';
			}
			if (!empty($model['city'])) {
				/* Prefers to have nationwide region instead of default if location is queried, but need to adapt for Canada
				$model['region'] = 'usa';
				*/
				$model['location'] = $model['city'] . '&radius=0';
			}
			if (!empty($model['zip'])) {
				/* Prefers to have nationwide region instead of default if location is queried, but need to adapt for Canada
				$model['region'] = 'usa';
				*/
				$model['location'] = $model['zip'] . '&radius=0';
			}
			if (!empty($model['subdivision'])) {
				if (isset($model['q'])) {
					$model['q'] .= ',' . $model['subdivision'];
				}
				else {
					$model['q'] = $model['subdivision'];
				}
			}
			if (!empty($model['keyword'])) {
				if (isset($model['q'])) {
					$model['q'] .= ',' . $model['keyword'];
				}
				else {
					$model['q'] = $model['keyword'];
				}
			}
			if (!empty($model['property_type'])) {
				if (!empty($model['category'])) {
					// Downcase
					$model['property_type'] = strtolower($model['property_type']);
					// Available to housing/sale & housing/rent
					if ($model['property_type'] == 'house') {
						$model['category'] .= '/home';
					}
					else if ($model['property_type'] == 'condo') {
						$model['category'] .= '/condo';
					}
					else if ($model['property_type'] == 'commercial') {
						$model['category'] .= '/commerical';
					}
					else if ($model['property_type'] == 'mobile') {
						$model['category'] .= '/mobile';
					}
					else if ($model['property_type'] == 'vacation') {
						$model['category'] .= '/vacation';
					}
					else if ($model['property_type'] == 'other') {
						$model['category'] .= '/other';
					}
					else if ($model['property_type'] == 'open_house') {
						$model['category'] .= '/open_house';
					}
					else if ($model['property_type'] == 'storage') {
						$model['category'] .= '/storage';
					}
					// Available only to housing/sale
					else if ($model['property_type'] == 'land') {
						$model['category'] = 'housing/sale/land';
					}
					else if ($model['property_type'] == 'multi') {
						$model['category'] = 'housing/sale/multi_family';
					}
					else if ($model['property_type'] == 'ranch') {
						$model['category'] = 'housing/sale/farm';
					}
					// Available only to housing/rent
					else if ($model['property_type'] == 'lease') {
						$model['category'] = 'housing/rent';
					}
					else if ($model['property_type'] == 'apartment') {
						$model['category'] = 'housing/rent/apartment';
					}
					else if ($model['property_type'] == 'garage') {
						$model['category'] = 'housing/rent/garage';
					}
					else if ($model['property_type'] == 'roommates') {
						$model['category'] = 'housing/rent/roommates';
					}
					else if ($model['property_type'] == 'short_term') {
						$model['category'] = 'housing/rent/short_term';
					}
				}
			}
			if ($model['pool_on_property'] == 'yes') {
				if (isset($model['amenities'])) {
					$model['amenities'] .= ',amenities_pool';
				}
				else {
					$model['amenities'] = 'pool';
				}
			}
			if ($model['is_gated_community'] == 'yes') {
				if (isset($model['amenities'])) {
					$model['amenities'] .= ',amenities_gated';
				}
				else {
					$model['amenities'] = 'gated';
				}
			}

			// Add each parameter from $parameters to the query URL when $model[parameter] = url_parameter for '&url_parameter='
			$parameters = array('region', 'category', 'location', 'q');
			foreach ($parameters as $parameter) {
				if (isset($model[$parameter])) {
					$query_url .= '&' . $parameter . '=' . str_replace(' ', '%20', $model[$parameter]);
				}	
			}

			// Add each Oodle attribute from $attributes to the query URl in the form of &attributes=all_attributes
			$attributes = array('price', 'bedrooms', 'bathrooms', 'square_feet', 'amenities');
			$first = true;
			foreach ($attributes as $attribute) {
				if (isset($model[$attribute])) {
					if ($first) {
						$query_url .= '&attributes=';
					}
					else {
						$query_url .= ',';
					}
					$query_url .= $attribute . '_' . $model[$attribute];
					$first = false;
				}	
			}
		}

		return $query_url;
	}
	public function get_listings($model) {
		$result = array();
		$json_array = wp_remote_get($model['query_url']);
		if (is_array($json_array)) {
			$json_data = json_decode($json_array['body']);
			if ($model['is_displet_api']) {
				$result['listings'] = $json_data->results;
				$result['meta'] = $json_data->meta;
				// Create first and last based on page
				$result['meta']->first = intval($model['listings_per_page']) * intval($result['meta']->page) - intval($model['listings_per_page']) + 1;
				$result['meta']->last = intval($result['meta']->first) + intval($model['listings_per_page']) - 1;
			}
			else{
				$result['listings'] = $json_data->listings;
				$result['meta'] = $json_data->meta;
				// Translate free API for variables used in tempalte
				$result['meta']->count = $result['meta']->total;
			}
		}
		return $result;
	}
	public function get_listing($model) {
		$result = array();
		if ($model['is_displet_api']) {
			$json_array = wp_remote_get('http://staging.api.displet.com/residentials/search?key=' . $model['app_key'] . '&sysid=' . $model['property_id']);
		}
		else{
			$json_array = wp_remote_get('http://api.oodle.com/api/v2/listings?key=579B95506556&format=json&jsoncallback=none&region=usa&q=' . $model['property_id']);
		}
		if (is_array($json_array)) {
			$json_data = json_decode($json_array['body']);

			// Create address & bathrooms
			if ($model['is_displet_api']) {
				if (!empty($json_data->results)) {
					$result['listings'] = $json_data->results;
					$result['listings'][0]->address = trim($result['listings'][0]->street_number . ' ' . $result['listings'][0]->street_name . ' ' . $result['listings'][0]->street_post_dir);
					$half_baths = (!empty($result['listings'][0]->half_baths)) ? '/' . $result['listings'][0]->half_baths : '';
					$result['listings'][0]->bathrooms = $result['listings'][0]->full_baths . $half_baths;
				}
			}

			// Translate variables for free version
			else {
				if (!empty($json_data->listings)) {
					$result['listings'] = $json_data->listings;
					$result['listings'][0]->list_price = $result['listings'][0]->attributes->price;
					$result['listings'][0]->address = $result['listings'][0]->location->address;
					$result['listings'][0]->city = $result['listings'][0]->location->name;
					$result['listings'][0]->state = $result['listings'][0]->location->state;
					$result['listings'][0]->zip = $result['listings'][0]->location->zip;
					$result['listings'][0]->num_bedrooms = $result['listings'][0]->attributes->bedrooms;
					$result['listings'][0]->bathrooms = $result['listings'][0]->attributes->bathrooms;
					$result['listings'][0]->square_feet = $result['listings'][0]->attributes->square_feet;
					$result['listings'][0]->property_type = $result['listings'][0]->category->name;
					$result['listings'][0]->mls_number = $result['listings'][0]->attributes->mls_id;
					$result['listings'][0]->internet_remarks = $result['listings'][0]->body;
					$result['listings'][0]->latitude = $result['listings'][0]->location->latitude;
					$result['listings'][0]->longitude = $result['listings'][0]->location->longitude;

					// Get largest of each photo
					if (is_array($result['listings'][0]->images)) {
						$images_array = array();
						// Get largest of each image
						for ($i = 0; $i < count($result['listings'][0]->images); $i++) {
							if ($result['listings'][0]->images[$i]->num == $last_num || $i == count($result['listings'][0]->images) - 1) {
								$last_src = '<span class="displet-image" style="background-image:url(' . $result['listings'][0]->images[$i]->src . ');"><img src="' . $result['listings'][0]->images[$i]->src . '" /></span>';
							}
							if (($result['listings'][0]->images[$i]->num > $last_num || $i == count($result['listings'][0]->images) - 1) && !empty($last_src)) {
								$images_array[] = $last_src;
							}
							$last_num = $result['listings'][0]->images[$i]->num;
						}
						$result['listings'][0]->large_image = $images_array[0];
						$result['listings'][0]->large_images = '';
						for ($i = count($images_array) - 1; $i >= 0; $i--) {
							$result['listings'][0]->large_images .= $images_array[$i];
						}
					}
				}
			}

			// Swap phones, emails and URLs from description
			if (!empty($result['listings'][0]->internet_remarks)) {
				$phone = '/([0-9]{3})[-. ]?([0-9]{4})/';
				$email = '/([\S]+)@([\S]+)/';
				$website = '/([\S]+)\.com([\S]+)/';
				$new_body = preg_replace($phone, 'XXX-XXXX', $result['listings'][0]->internet_remarks);
				$new_body = preg_replace($email, 'XXXX@XXXX', $new_body);
				$new_body = preg_replace($website, 'http://.com', $new_body);
				$result['listings'][0]->internet_remarks = $new_body;
			}

			// Create maps from lat/long & address
			if ($result['listings'][0]->latitude && $result['listings'][0]->longitude && $result['listings'][0]->address && $result['listings'][0]->city && $result['listings'][0]->state) {
				$result['listings'][0]->maps = '<iframe class="displet-map-iframe" width="100%" height="300" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="https://maps.google.com/maps?q=' . $result['listings'][0]->address . ',+' . $result['listings'][0]->city . ',+' . $result['listings'][0]->state . '&amp;cbp=13,15.23,,0,0&amp;t=m&amp;z=14&amp;iwloc=A&amp;source=embed&amp;output=svembed"></iframe>';
				$result['listings'][0]->maps .= '<iframe class="displet-street-view-iframe" width="100%" height="300" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="https://maps.google.com/maps?q=' . $result['listings'][0]->address . ',+' . $result['listings'][0]->city . ',+' . $result['listings'][0]->state . '&amp;layer=c&amp;cbp=13,15.23,,0,0&amp;cbll=' . $result['listings'][0]->latitude . ',' . $result['listings'][0]->longitude . '&amp;t=m&amp;z=14&amp;iwloc=A&amp;source=embed&amp;output=svembed"></iframe>';
			}
		}
		return $result;
	}
}

?>