<?php

class DispletRetsIdxUsers{
	public function register_user_ajax(){
		// Check nonce for security
		check_ajax_referer('displet_register_user_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_user_registration_request'){
			if (!empty($_POST['name']) && !empty($_POST['email'])) {
				if (is_email($_POST['email'])) {
					$user_registration_response = self::register_user($_POST['email'], $_POST['name'], $_POST['phone'], $_POST['url']);
					echo $user_registration_response;
				}
				else{
					echo 'Please enter a valid email address.';
				}
			}
			else{
				echo 'Please complete the required fields.';
			}
			die();
		}

		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function register_user($email, $name, $phone = '', $url = '', $upstream_url = '', $last_hash = '') {
		$admin_email = DispletRetsIdxEmails::get_admin_email_address();

		// Create WP User
		$random_password = wp_generate_password($length=12, $include_standard_special_chars=false);
		$user_id = wp_create_user($email, $random_password, $email);
		if (is_wp_error($user_id)) {
			return $user_id->get_error_message();
		}

		// Set admin as email sender
		add_filter('wp_mail_from', array('DispletRetsIdxEmails', 'set_admin_email_as_sender'));
		add_filter('wp_mail_from_name',array('DispletRetsIdxEmails', 'set_admin_name_as_sender'));

		// Message to admin
		$admin_message =
			'User Name: ' . $name . "\r\n" .
			'User Email: ' . $email . "\r\n" .
			'User Phone: ' . $phone . "\r\n" .
			'Registration Page: ' . $url;
		wp_mail($admin_email, 'A New User Has Registered at ' . site_url(), $admin_message);

		// Message to user
		$user_message =
			'Thank you for registering at ' . site_url() . '.' . "\r\n" .
			'Name: ' . $name . "\r\n" .
			'Email: ' . $email . "\r\n" .
			'Phone: ' . $phone . "\r\n" .
			'Password: ' . $random_password;
		wp_mail($email, 'New Account Details for ' . site_url(), $user_message);
		
		// Remove admin as email sender
		remove_filter('wp_mail_from', array('DispletRetsIdxEmails', 'set_admin_email_as_sender'));
		remove_filter('wp_mail_from_name',array('DispletRetsIdxEmails', 'set_admin_name_as_sender'));

		// Login user
		$creds = array(
			'user_login' => get_userdata($user_id)->user_login,
			'user_password' => $random_password,
			'remember' => true
		);
		$user = wp_signon($creds, false);

		// Assign user meta
		update_user_meta($user_id, 'displet_lead', true);
		update_user_meta($user_id, 'nickname', $name);
		update_user_meta($user_id, 'displet_phone', $phone);
		update_user_meta($user_id, 'displet_registration_url', $url);
		update_user_meta($user_id, 'displet_registration_upstream_url', $upstream_url);
		update_user_meta($user_id, 'displet_last_hash', $last_hash);

		// Send to Zapier
		$zap_user = array(
			'email' => $email,
			'name' => $name,
			'phone' => $phone,
			'registered_at' => $url,
			'came_from' => $upstream_url
		);
		self::send_to_zapier($zap_user, 'registration');

		return 'Successful Registration';
	}

	public function login_user_ajax() {
		// Check nonce for security
		check_ajax_referer('displet_login_user_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_rets_idx_user_signon'){
			if (!empty($_POST['email']) && !empty($_POST['password'])) {
				$creds = array();
				$creds['user_login'] = $_POST['email'];
				$creds['user_password'] = $_POST['password'];
				$creds['remember'] = true;
				$user = wp_signon($creds, false);
				if (is_wp_error($user)) {
					echo $user->get_error_message();
				}
				else{
					echo 'Successful Login';
				}
			}
			else {
				echo 'Please complete the required fields.';
			}
			die();
		}
		
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function check_user_ajax() {
		// Check nonce for security
		check_ajax_referer('displet_check_user_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_user_check_request'){
			if (!empty($_POST['email'])) {
				$user_exists = self::check_user($_POST['email']);
				if ($user_exists) {
					echo 'User Exists and Logged In';
				}
			}
			else {
				echo 'There is no email account associated with this user.';
			}
			die();
		}
		
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function check_user($email){
		$user_id = username_exists($email);
		if (!empty($user_id)) {
			wp_set_auth_cookie($user_id, true);
			return true;
		}
		else{
			return false;
		}
	}

	public function check_login_ajax(){
		// Check nonce for security
		check_ajax_referer('displet_check_login_nonce');

		if (is_user_logged_in()) {
			echo 'Successful Login';
		}
		else{
			echo 'User Not Logged In';
		}
		die();
	}

	public function update_users_last_login($user_login, $user){
		update_user_meta($user->ID, 'displet_last_login', date('Y-m-j h:i:s'));
	}

	public function delete_users(){
		// Check nonce for security
		check_ajax_referer('displet_delete_users_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_delete_users_request'){
			if (!empty($_POST['displet_users'])) {
				$error_occurred = false;
				foreach ($_POST['displet_users'] as $user){
					$deleted[] = wp_delete_user($user);
				}
				foreach ($deleted as $was_deleted) {
					if (!$was_deleted){
						$error_occurred = true;
					}
				}
				if ($error_occurred){
					echo 'There was an error processing your request.';
				}
				else{
					echo 'Succesful Deletion';
				}
			}
			die();
		}
	}

	public function hide_user_bar_from_leads(){
		$user = wp_get_current_user();
		if (get_user_meta($user->ID, 'displet_lead', true) != '') {
			show_admin_bar(false);
		}
	}

	public function send_to_zapier($user_data, $zap){
		$displet_rets_idx_option = DispletRetsIdxSettings::get_option();
		if (!empty($displet_rets_idx_option['displet_rets_idx_option_zapier_' . $zap . '_url'])) {
			$request = array(
				'user-agent' => 'Displet',
				'body' => $user_data,
				'referer' => site_url()
			);
			$urls = explode("\n", $displet_rets_idx_option['displet_rets_idx_option_zapier_' . $zap . '_url']);
			foreach ($urls as $url) {
				$post_url = trim($url);
				if (!empty($post_url)) {
					wp_remote_post($post_url, $request);
				}
			}
		}
	}
}

add_action('wp_ajax_displet_user_registration_request', array('DispletRetsIdxUsers', 'register_user_ajax'));
add_action('wp_ajax_nopriv_displet_user_registration_request', array('DispletRetsIdxUsers', 'register_user_ajax'));
add_action('wp_ajax_displet_rets_idx_user_signon', array('DispletRetsIdxUsers', 'login_user_ajax'));
add_action('wp_ajax_nopriv_displet_rets_idx_user_signon', array('DispletRetsIdxUsers', 'login_user_ajax'));
add_action('wp_ajax_displet_user_check_request', array('DispletRetsIdxUsers', 'check_user_ajax'));
add_action('wp_ajax_nopriv_displet_user_check_request', array('DispletRetsIdxUsers', 'check_user_ajax'));
add_action('wp_ajax_displet_check_login_request', array('DispletRetsIdxUsers', 'check_login_ajax'));
add_action('wp_ajax_nopriv_displet_check_login_request', array('DispletRetsIdxUsers', 'check_login_ajax'));
add_action('wp_ajax_displet_delete_users_request', array('DispletRetsIdxUsers', 'delete_users'));
add_action('wp_login', array('DispletRetsIdxUsers', 'update_users_last_login'), 10, 2);	
add_action('init', array('DispletRetsIdxUsers', 'hide_user_bar_from_leads'));	

?>