<?php

class DispletRetsIdxListings{
	function render_shortcode($attributes){
		// Retrieve options from settings page
		$displet_rets_idx_option = DispletRetsIdxSettings::get_option();
	
		// Create list of acceptable attributes (array keys) and a $model['key'] from each array key, using the shortcode attribute as the value when specified, else the setting default or other specification in array
		$model = shortcode_atts(array(
			// Has settings defaults
			'is_displet_api' => (!empty($displet_rets_idx_option['displet_rets_idx_option_displet_app_key'])) ? true : false,
			'map_variance' => stripslashes($displet_rets_idx_option['displet_rets_idx_option_map_variance']),
			'region' => stripslashes($displet_rets_idx_option['displet_rets_idx_option_oodle_region']),
			'category' => stripslashes($displet_rets_idx_option['displet_rets_idx_option_oodle_category']),
			'listings_per_page' => intval($displet_rets_idx_option['displet_rets_idx_option_listings_per_page']),
			'orientation' => stripslashes($displet_rets_idx_option['displet_rets_idx_option_listings_orientation']),
			'default_sort' => stripslashes($displet_rets_idx_option['displet_rets_idx_option_listings_sort']),
			'sort_by' => stripslashes($displet_rets_idx_option['displet_rets_idx_option_sort_by']),
			'sort_order' => stripslashes($displet_rets_idx_option['displet_rets_idx_option_sort_order']),
			'return_fields' => 'city,full_baths,half_baths,image_urls,internet_remarks,latitude,list_price,listing_office_name,longitude,mls_number,num_bedrooms,property_type,square_feet,state,street_name,street_number,street_post_dir,street_pre_dir,subdivision,sysid,zip',
			// Has only shortcode attributes
			'caption' => '',
			'minlistprice' => '',
			'maxlistprice' => '',
			'minbedrooms' => '',
			'maxbedrooms' => '',
			'minbathrooms' => '',
			'maxbathrooms' => '',
			'minsquarefeet' => '',
			'maxsquarefeet' => '',
			'area' => '', // IDX only (user-defined area)
			'area_mls_defined' => '',
			'city' => '',
			'property_type' => '',
			'minstories' => '', // IDX only
			'waterfront' => '', // IDX only
			'pool_on_property' => '', 
			'subdivision' => '', // IDX only
			'keyword' => '',
			'school' => '', // IDX only
			'school_district' => '', // IDX only
			'minacres' => '', // IDX only
			'zip' => '',
			'is_foreclosure' => '',
			'yearbuilt' => '', // IDX only - possible to add exact match year for Displet Lite
			'status' => '', // IDX only
			'short_sale' => '', // IDX only
			'last_modified' => '', // IDX only - wasn't there on IDX
			'listing_agent_id' => '', // IDX only
			'listing_office_id' => '', // IDX only
			'min_lot_size' => '', // IDX only - wasn't there on IDX
			'is_gated_community' => '',
			'min_sold_date' => '', // IDX only - wasn't there on IDX
			'max_sold_date' => '', // IDX only - wasn't there on IDX
			'master_on_main' => '', // IDX only - wasn't there on IDX
			'basement' => '', // IDX only - wasn't there on IDX
			'min_square_feet_available' => '', // IDX only - wasn't there on IDX
			'max_square_feet_available' => '', // IDX only - wasn't there on IDX
			'min_price_per_sq_feet' => '', // IDX only - wasn't there on IDX
			'max_price_per_sq_feet' => '', // IDX only - wasn't there on IDX
		), $attributes);
	
		// Get query from $model
		$model['query_url'] = DispletRetsIdxQuery::get_query_url($model);
	
		// Get current page and create previous/next links, update start number if paginated
		$page_id = intval(get_query_var('paged'));
		$page_url = get_permalink();
		if ($page_id) {
			$next_page_id = $page_id + 1;
			$previous_page_id = $page_id - 1;
			if ($page_id < 2) {
				$previous_page = '/#previous';
			}
			else{
				$previous_page = '/page/' . $previous_page_id;
			}
			$next_page = '/page/' . $next_page_id;
			$model['previous_page_url'] = rtrim($page_url, '/') . $previous_page;
			$model['next_page_url'] = rtrim($page_url, '/') . $next_page;
			if ($page_id > 1) {
				if ($model['is_displet_api']) {
					$new_start = '&page=' . $page_id;
					$model['query_url'] = str_replace('&page=1', $new_start, $model['query_url']);
				}
				else{
					$start_id = ($page_id - 1) * $model['listings_per_page'] + 1;
					$new_start = '&start=' . $start_id;
					$model['query_url'] = str_replace('&start=1', $new_start, $model['query_url']);
				}
			}
		}
		else{
			$model['previous_page_url'] = rtrim($page_url, '/') . '/#previous';
			$model['next_page_url'] = rtrim($page_url, '/') . '/page/2';
		}
	
		// Get listings server-side to pass to templates for crawlable content
		$listings = DispletRetsIdxQuery::get_listings($model);
		$model = array_merge($model, $listings);
	
		// Grab appropriate templates
		$displet_templates = '';
		if (!empty($model['caption'])) {
			$displet_templates .= '<h2>' . $model['caption'] . '</h2>';	
		}
		if (intval($model['meta']->count) > 0) {
			$displet_templates .= DispletRetsIdxLoad::get_template('displet-listings-jquery.php', $model);
			$displet_templates .= DispletRetsIdxLoad::get_template('displet-map-jquery.php', $model);
			if ($model['is_displet_api']) {
				if (intval($model['meta']->count) > 9) {
					$displet_templates .= DispletRetsIdxLoad::get_template('displet-price-navigation.php', $model);
				}
				$displet_templates .= DispletRetsIdxLoad::get_template('displet-statistics.php', $model);
			}
		}
		$displet_templates .= DispletRetsIdxLoad::get_template('displet-dynamic.php', $model);
	
		return $displet_templates;
	}
	
	function update_canonical(){
		// Determine if page contains DispletListing shortcode
		$shortcode = false;
		$regex = '/' . get_shortcode_regex() . '/s';
		global $posts;
		if (isset($posts[0])) {
			preg_match($regex, $posts[0]->post_content, $matches);
			if (is_array($matches) && isset($matches[2]) && $matches[2] == 'DispletListing') {
				$shortcode = true;
			}
		}
		// Remove default canonical and add ours
		if (get_query_var('paged') && $shortcode){
			remove_action('wp_head', 'rel_canonical');
			add_action('wp_head', array('DispletRetsIdxListings', 'add_canonical'), 1);
		}
	}

	function add_canonical(){
		// Add canonical that includes /page/id
		$link = rtrim(get_permalink(), '/');
		$paged = get_query_var('paged');
		if ($paged) {
			$link .= '/page/' . $paged;
		}
	    echo '<link rel="canonical" href="' . esc_url($link) . '" />';
	}

	function remove_yoasts_canonical($canonical) {
		// Determine if page contains DispletListing shortcode
		$shortcode = false;
		$regex = '/' . get_shortcode_regex() . '/s';
		global $posts;
		if (isset($posts[0])) {
			preg_match($regex, $posts[0]->post_content, $matches);
			if (is_array($matches) && isset($matches[2]) && $matches[2] == 'DispletListing') {
				$shortcode = true;
			}
		}
		if (get_query_var('paged') && $shortcode) {
	    	$canonical = false;
	    }
		return $canonical;
	}
}

add_shortcode('DispletListing', array('DispletRetsIdxListings', 'render_shortcode'));
add_action('template_redirect', array('DispletRetsIdxListings', 'update_canonical'));
add_filter('wpseo_canonical', array('DispletRetsIdxListings', 'remove_yoasts_canonical'));

?>